
#define ROLL       0
#define PITCH      1
#define YAW        2

#define I2C_SPEED 100000L     //100kHz normal mode, this value must be used for a genuine WMP
//#define I2C_SPEED 400000L   //400kHz fast mode, it works only with some WMP clones

//enable internal I2C pull ups
//#define INTERNAL_I2C_PULLUPS

// for Arduino MEGA
#define I2C_PULLUPS_ENABLE         PORTD |= 1<<0; PORTD |= 1<<1;       // PIN 20&21 (SDA&SCL)
#define I2C_PULLUPS_DISABLE        PORTD &= ~(1<<0); PORTD &= ~(1<<1);

//#include <avr/io.h>
#include <TwiMaster.h>

TwiMaster rtc;

static uint8_t rawADC_NUN[6];
static int16_t accADC[3];


void setup ()
{
  Serial.begin (115200);
  delay(100);

  // true to enable pullups
  rtc.init(false);
  delay(100);

  i2c_ACC_init();

  Serial.println("\nFinished setup");
}


void loop ()
{
  i2c_ACC_getADC();

  Serial.print("\nX: "); 
  Serial.print(rawADC_NUN[2], DEC);
  Serial.print("\tY: "); 
  Serial.print(rawADC_NUN[3], DEC);
  Serial.print("\tZ: "); 
  Serial.print(rawADC_NUN[4], DEC);

  Serial.print("\nROLL: "); 
  Serial.print(accADC[ROLL], DEC);
  Serial.print("\tPITCH: "); 
  Serial.print(accADC[PITCH], DEC);
  Serial.print("\tYAW: "); 
  Serial.print(accADC[YAW], DEC);

  delay (500);
}


void i2c_ACC_init() {
  //I2C_PULLUPS_DISABLE
  
  //TWBR = ((16000000L / I2C_SPEED) - 16) / 2;

  // disable encryption
  // look at <http://wiibrew.org/wiki/Wiimote#The_New_Way> at "The New Way"
  rtc.start(0xA4 + 0);//I2C write direction => 0
  rtc.write(0xF0); 
  rtc.write(0x55);
  rtc.stop();
  
  rtc.start(0xA4 + 0);//I2C write direction => 0
  rtc.write(0xFB); 
  rtc.write(0x00);
  rtc.stop();
  
  
//  look at <http://wiibrew.org/wiki/Wiimote#The_Old_Way> at "The Old Way"
//  rtc.start(0xA4 + 0);//I2C write direction => 0
//  rtc.write(0x40); 
//  rtc.write(0x00);
//  rtc.stop();

  delay(10);

  // display the identification bytes, must be "00 00 A4 20 00 00" for the Nunchuck
  rtc.start(0xA4 + 0);//I2C write direction => 0
  rtc.write(0xFA);
  rtc.stop();
  delay(10);
  
  rtc.start(0xA4 + 1);//I2C read direction => 1
  for(uint8_t i = 0; i < 5; i++) {
    rawADC_NUN[i]=rtc.readAck();
  }  
  rawADC_NUN[5]= rtc.readNak();
  rtc.stop();

  Serial.print("\n\nIdent=");
  for (byte i = 0; i < 6; i++)
  {
    Serial.print(rawADC_NUN[i], HEX);
    Serial.print(' ');
  }
  Serial.println();

}

void i2c_ACC_getADC() {
  //TWBR = ((16000000L / I2C_SPEED) - 16) / 2; // change the I2C clock rate. !! you must check if the nunchuk is ok with this freq

  rtc.start(0xA4 + 0);//I2C write direction => 0
  rtc.write(0x00);
  rtc.stop();
  delay(1);

  rtc.start(0xA4 + 1);//I2C read direction => 1
  for(uint8_t i = 0; i < 5; i++)
    rawADC_NUN[i]=rtc.readAck();
  rawADC_NUN[5]= rtc.readNak();
  rtc.stop();
 
  accADC[ROLL]  =   ( (rawADC_NUN[3]<<2)        + ((rawADC_NUN[5]>>4)&0x2) );
  accADC[PITCH] = - ( (rawADC_NUN[2]<<2)        + ((rawADC_NUN[5]>>3)&0x2) );
  accADC[YAW]   = - ( ((rawADC_NUN[4]&0xFE)<<2) + ((rawADC_NUN[5]>>5)&0x6) );
  
}

